/*
 * NumberComparator.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import java.util.Comparator;


/**
 * A <code>Comparator</code> that compares numbers.
 * <br />
 * <br />
 * This class provides various <code>compare</code> methods, a series which
 * cascades down from generic type to the specific number type
 * <br />
 * <code>Object</code> ==> <code>Number</code> ==>
 * <code>long</code>&nbsp;or&nbsp;<code>double</code>
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.0
 * @see java.util.Comparator
 */
public class NumberComparator implements Comparator
{
	
	/*
	 * Exported constants
	 */
	
	/** A <code>NumberComparator</code> to sort numbers lowest to highest. */
	public static final NumberComparator ASCENDING = new NumberComparator( false );
	
	/** A <code>NumberComparator</code> to sort numbers highest to lowest. */
	public static final NumberComparator DESCENDING = new NumberComparator( true );
	
	
	/*
	 * Instance variables
	 */
	
	/** Indicates if the ordering of numbers should be reversed, thus sorting
	 *  numbers into descending order. Descending is considered to be 'reverse'
	 *  since when this flag is set the values returned by the compare method
	 *  will be flipped - a positive value (which usually indicates a bigger
	 *  number) will instead indicate a smaller number. */
	protected boolean _descending;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * The default constructor.
	 * <br />
	 * Creates a comparator that will sort numbers in ascending order.
	 */
	protected NumberComparator()
	{
		_descending = false;
	}
	
	/**
	 * Create a comparator that will sort numbers.
	 * <br />
	 * <br />
	 * @param descending if set to <code>false</code> numbers will be in
	 *                   ascending order; if set to <code>true</code> files will
	 *                   be sorted in descending order
	 */
	protected NumberComparator( boolean descending )
	{
		_descending = descending;
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Compares two <code>Number</code>s for order. Returns a negative integer,
	 * zero, or a positive integer as the first number is less than, equal
	 * to, or greater than the second.
	 * <br />
	 * <br />
	 * This implementation ensures that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * This implementation also ensures that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * This implementation also ensures that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * Finally, this implementation also ensures that
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
	 * <br />
	 * <br />
	 * @param number1 the first number to be compared
	 * @param number2 the second number to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first number is less than, equal to, or greater than the
	 *	       second
	 */
	public int compare( Number number1, Number number2 )
	{
		boolean anyFloatingPoint = (   (number1 instanceof Float)
		                            || (number2 instanceof Float)
		                            || (number1 instanceof Double)
		                            || (number2 instanceof Double)  );
		
		if (anyFloatingPoint)
			return compare( number1.doubleValue(), number2.doubleValue() );
		else
			return compare( number1.longValue(), number2.longValue() );
	}
	
	/**
	 * Compares two floating-point numbers for order. Returns a negative
	 * integer, zero, or a positive integer as the first number is less than,
	 * equal to, or greater than the second.
	 * <br />
	 * <br />
	 * This implementation ensures that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * This implementation also ensures that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * This implementation also ensures that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * Finally, this implementation also ensures that
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
	 * <br />
	 * <br />
	 * @param number1 the first number to be compared
	 * @param number2 the second number to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first number is less than, equal to, or greater than the
	 *	       second
	 */
	public int compare( double number1, double number2 )
	{
		// Focus on ascending case.
		// The diff should be positive if number1 is greater than number2.
		double basicDiff = number1 - number2;
		
		// a simple cast (see code below) may result in overflow
		// which may cause the int value to flip sign
		//int diff = (int)basicDiff;
		
		// so check the bounds of int before doing the cast
		int diff;
		if (basicDiff < Integer.MIN_VALUE)
		{
			// NB: Remember: we may negate the result if this comparator is in
			//     descending mode. Since abs( min ) = abs( max ) + 1, we need
			//     to adjust the min value by one such that if negated it would
			//     not overflow the max value.
			diff = Integer.MIN_VALUE + 1;
		}
		else if (basicDiff > Integer.MAX_VALUE)
		{
			diff = Integer.MAX_VALUE;
		}
		else
		{
			diff = (int)basicDiff;
		}
		
		// check for rounding which makes a small diff become zero
		if (diff == 0)
		{
			if (basicDiff > 0.0)
				diff = 1;
			if (basicDiff < 0.0)
				diff = -1;
		}
		
		// now reverse (negate) the result if necessary
		return (_descending) ? -diff : diff;
	}
	
	/**
	 * Compares two fixed-point numbers for order. Returns a negative integer,
	 * zero, or a positive integer as the first number is less than, equal
	 * to, or greater than the second.
	 * <br />
	 * <br />
	 * This implementation ensures that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * This implementation also ensures that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * This implementation also ensures that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * Finally, this implementation also ensures that
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
	 * <br />
	 * <br />
	 * @param number1 the first number to be compared
	 * @param number2 the second number to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first number is less than, equal to, or greater than the
	 *	       second
	 */
	public int compare( long number1, long number2 )
	{
		// Focus on ascending case.
		// The diff should be positive if number1 is greater than number2.
		long basicDiff = number1 - number2;
		
		// a simple cast (see code below) may result in overflow
		// which may cause the int value to flip sign
		//int diff = (int)basicDiff;
		
		// so check the bounds of int before doing the cast
		int diff;
		if (basicDiff < Integer.MIN_VALUE)
		{
			// NB: Remember: we may negate the result if this comparator is in
			//     descending mode. Since abs( min ) = abs( max ) + 1, we need
			//     to adjust the min value by one such that if negated it would
			//     not overflow the max value.
			diff = Integer.MIN_VALUE + 1;
		}
		else if (basicDiff > Integer.MAX_VALUE)
		{
			diff = Integer.MAX_VALUE;
		}
		else
		{
			diff = (int)basicDiff;
		}
		
		// now reverse (negate) the result if necessary
		return (_descending) ? -diff : diff;
	}
	
	
	/*
	 * Implementation of Comparator interface
	 */
	
	public int compare( Object o1, Object o2 )
	{
		return compare( (Number)o1, (Number)o2 );
	}
	
	public boolean equals( Object obj )
	{
		if (obj instanceof NumberComparator)
		{
			NumberComparator other = (NumberComparator)obj;
			
			// equal if same ordering
			return (_descending == other._descending);
		}
		else
		{
			// not a NumberComparator, so can't be equal
			return false;
		}
	}
	
}


/*
 * Change History
 *
 * $Log: NumberComparator.java,v $
 * Revision 1.1  2006/09/13 20:09:05  Steve
 * Initial revision.
 *
 */