/*
 * Fonts.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.gui;

import java.util.StringTokenizer;
import java.awt.Font;


/**
 * Utility class for operations with <code>Font</code>s.
 * <br />
 * <br />
 * @see java.awt.Font
 * @author Stephen Battey
 * @version $Revision: 1.4 $
 * @since 1.0
 */
public class Fonts
{
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * Default constructor.
	 * Hidden (made protected) since this is a static class.
	 * <br />
	 * <br />
	 * @since 1.0
	 */
	protected Fonts()
	{
	}


	/*
	 * Static methods
	 */

	/**
	 * Get a font with one of the names (in order of preference) and the
	 * specified style & size.
	 * <br />
	 * <br />
	 * @param names A list of font names separated by commas.
	 *              These can be a logical font name or a font face name. A
	 *              logical name must be either: Dialog, DialogInput,
	 *              Monospaced, Serif, or SansSerif.
	 *              First a font with the first name in the list will be
	 *              created. If this font cannot be found then the second name
	 *              will be used and so on until the list is exhausted.
	 *              If there is no valid font specified in the list or if the
	 *              <code>name</code> is <code>null</code>, then the
	 *              <code>name</code> of the new <code>Font</code> is set to the
	 *              name "Default".
	 * @param style The style constant for the <code>Font</code>.
	 *              The style argument is an integer bitmask that may be PLAIN,
	 *              or a bitwise union of BOLD and/or ITALIC (for example,
	 *              ITALIC or BOLD|ITALIC). If the style argument does not
	 *              conform to one of the expected integer bitmasks then the
	 *              style is set to PLAIN.
	 * @param size the point size of the <code>Font</code>
	 * @return Font
	 * @since 1.0
	 */
	public static final Font getFont( String names, int style, int size )
	{
		Font font = null;
		
		/* guard against null names list */
		if (names != null)
		{
			/* parse each font name in the list */
			StringTokenizer fontNameTokeniser = new StringTokenizer( names, ",", false );
			
			while (   (font == null)
			       && (fontNameTokeniser.hasMoreTokens())  )
			{
				/* attempt to create a font with this name */
				String fontName = fontNameTokeniser.nextToken().trim();
				Font testFont = new Font( fontName, style, size );
				if (testFont.getFamily().equalsIgnoreCase( fontName ))
					font = testFont;
			}
		}
		
		if (font == null)
			font = new Font( null, style, size );
		
		return font;
	}
	
}


/*
 * Change History
 *
 * $Log: Fonts.java,v $
 * Revision 1.4  2006/09/06 21:07:53  Steve
 * Added legal disclaimer.
 *
 * Revision 1.3  2006/06/07 20:09:27  Steve
 * Fixed JavaDoc comment.
 *
 * Revision 1.2  2006/06/07 20:08:45  Steve
 * Brought into Bam library:
 *  - moved into lucidviews package
 *  - added CVS headers/footers
 *
 */
