/*
 * MouseActivityNotifier.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.gui.event;

import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;


/**
 * Listens for mouse events and sends notification when the user is using or has
 * stopped using the mouse.
 * <br />
 * <br />
 * For each mouse event, if the notifier is not already in an active state, an
 * active event is published. A thread is then started which causes a delay
 * before sending the dormant event. If another mouse event received within the
 * delayed time period slot then another thread is started to re-start the
 * delay.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public class MouseActivityNotifier implements MouseMotionListener
{
	
	/*
	 * Instance variables
	 */
	
 	protected Collection                 _listeners;
 	protected long                       _delay;
 	protected InactiveNotificationThread _inactiveNotificationThread;
 	
 	
 	/*
 	 * Constructor(s)
 	 */
 	
 	/**
 	 * Create a mouse activity notifier that notifies all of its listeners when
 	 * mouse activity begins and when no activity has been received for the given
 	 * time period.
 	 * <br />
 	 * <br />
 	 * @param delay the number of milli-seconds after which the mouse is
 	 *              considered inactive
 	 */
 	public MouseActivityNotifier( long delay )
 	{
 		_listeners = new ArrayList();
 		_delay = delay;
 		_inactiveNotificationThread = null;
 	}
 	
 	
 	/*
 	 * Implementation of MouseMotionListener interface
 	 */
 	
 	public void mouseDragged( MouseEvent e )
 	{
 		notifyMouseNowActive( e );
 		startInactiveNotificationThread();
 	}
 	
 	public void mouseMoved( MouseEvent e )
 	{
 		notifyMouseNowActive( e );
 		startInactiveNotificationThread();
 	}
 	
 	
 	/*
 	 * Implementation of MouseActivityListener notifier
 	 */
 	
 	/**
 	 * Register a listener that wants to be notified of mouse activity.
 	 * <br />
 	 * <br />
 	 * @param listener object that is to receive notifications
 	 */
 	public void addIsolatedKeyEventListener( MouseActivityListener listener )
 	{
 		synchronized (_listeners)
 		{
 			if (!_listeners.contains( listener ))
 				_listeners.add( listener );
 		}
 	}
 	
 	/**
 	 * Un-register a listener that does not want to be notified of mouse activity.
 	 * <br />
 	 * <br />
 	 * @param listener object that is no longer to receive notifications
 	 */
 	public void removeIsolatedKeyEventListener( MouseActivityListener listener )
 	{
 		synchronized (_listeners)
 		{
 	    if (_listeners.contains( listener ))
 	    	_listeners.remove( listener );
 		}
 	}
 	
 	/**
 	 * Notify all listeners that the mouse is now active.
 	 * <br />
 	 * <br />
 	 * @param mouseEvent the mouse event that was received
 	 */
 	protected void notifyMouseNowActive( MouseEvent mouseEvent )
 	{
 		/* notify all listeners of the activation */
 		synchronized (_listeners)
 		{
 			Iterator itr = _listeners.iterator();
 			while (itr.hasNext())
 				((MouseActivityListener)itr.next()).mouseNowActive( mouseEvent );
 		}
 	}
 	
 	/**
 	 * Notify all listeners that the mouse is now inactive.
 	 * <br />
 	 * <br />
 	 */
 	protected void notifyMouseNowInactive()
 	{
 		/* notify all listeners of the inactive state */
 		synchronized (_listeners)
 		{
 			Iterator itr = _listeners.iterator();
 			while (itr.hasNext())
 				((MouseActivityListener)itr.next()).mouseNowInactive();
 		}
 	}
 	
 	
 	/*
 	 * Methods
 	 */

 	/**
 	 * Tells this activity notifier to listen to the specified component for mouse
 	 * events.
 	 * <br />
 	 * <br />
 	 * @param component the component from which mouse events indicate mouse
 	 *                  activity
 	 */
 	public void addSource( Component component )
 	{
 		component.addMouseMotionListener( this );
 	}
 	
 	/**
 	 * Tells this activity notifier to stop listening to the specified component
 	 * for mouse events.
 	 * <br />
 	 * <br />
 	 * @param component the component from which mouse events no longer indicate
 	 *                  mouse activity
 	 */
 	public void removeSource( Component component )
 	{
 		component.removeMouseMotionListener( this );
 	}
 	
 	/**
 	 * Start a new thread that will call the inactive method after the specified
 	 * delay.
 	 * <br />
 	 * <br />
 	 */
 	protected synchronized void startInactiveNotificationThread()
 	{
 		if (_inactiveNotificationThread != null)
 			/* any previous thread is now superseded */
 			_inactiveNotificationThread._superseded = true;
 		
 		/* start a new thread, invoking a delay from this moment */
 		_inactiveNotificationThread= new InactiveNotificationThread( this, _delay );
 		_inactiveNotificationThread.start();
 	}
 	
 	
 	/*
 	 * Helper class
 	 */
 	
 	public static class InactiveNotificationThread extends Thread
 	{
 		
 		/*
 		 * Instance variables
 		 */
 		
 		protected MouseActivityNotifier _mouseActivityNotifier;
 		protected long                  _delay;
 		protected boolean               _superseded;
 		
 		
 		/*
 		 * Constructor(s)
 		 */
 		
 		public InactiveNotificationThread( MouseActivityNotifier mouseActivityNotifier,
 		                                   long delay )
 		{
 			_mouseActivityNotifier = mouseActivityNotifier;
 			_delay = delay;
 			_superseded = false;
 		}
 		
 		
 		/*
 		 * Overridden methods of Thread base class
 		 */
 		
 		public void run()
 		{
 			/* pause for the desired delay */
 			try
 			{
 				Thread.sleep( _delay );
 			}
 			catch (InterruptedException e)
 			{
 			}
 			
 			if (!_superseded)
 				_mouseActivityNotifier.notifyMouseNowInactive();
 		}
 		
 	}
 	
}


/*
 * Change History
 *
 * $Log: MouseActivityNotifier.java,v $
 * Revision 1.3  2006/09/06 21:07:54  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/07 20:41:10  Steve
 * Brought into Bam library:
 *  - moved into lucidviews package
 *  - added CVS headers/footers
 *
 */
